#include <dic.hxx>
#ifdef WIN32
#include <process.h>
#include <windows.h>
#include <tlhelp32.h>
int getppid();
int init_sock();
#endif
#include "mcserver.h"

/************************************************************************/
/*			CREATE_ENV_SERV					*/
/*									*/
/*  Description: This function gets the value of the enviroment 	*/
/*		 variable given by "name" and stores it in the memory	*/
/*		 position given by data.				*/
/*									*/
/*  Parameters:  name -> Name of the enviroment variable that contains	*/
/*			 the data to copy to the position of "data" 	*/
/*									*/
/*		 data -> VOID pointer pointing the position where the 	*/
/*			 function should copy the value get from "name"	*/
/*									*/
/*		 type -> This gives to the function the type of data	*/
/*			 that is contained in the ENV variable. Could 	*/
/*			 be: I =integer, C= char string, L= long	*/
/*									*/
/*  Return Value: 0 if everything goes OK.				*/
/*		  -1 if there is any problem.				*/
/*									*/
/************************************************************************/

int create_env_serv(char *name, void *data, char type)
{
	char *pdata, *ptr, auxdata[128];
	
	pdata=getenv(name);
	
	if(pdata==NULL)
	{
		if(type=='C')
			strcpy((char *)data,"none");
		else
			*(int *)data=0;
		return(0);
	}
	
	//	if(pdata==0)
	//	return(-1);
	
		
	if( type=='I')
	{
		if(!strcmp(name,"mcpdf") || !strcmp(name,"mcseed"))
		{
			strcpy(auxdata,pdata);
			ptr = auxdata;
			while(*ptr!=' ')
				ptr++;
			*ptr='\0';
			
			*(int *)data=atoi(auxdata);
			*(int *)((char *)data+sizeof(int))=atoi(ptr+1);
		}
		else
			*(int *)data=atoi(pdata);
	}
	else if(type=='L')
		*(long *)data=atol(pdata);	
	else if( type=='C')
	  {
	    if(!strcmp(name,"DGJOBID"))
	      {
		strcpy(auxdata,pdata);
		if(ptr = strchr(auxdata,'.'))
		  *ptr = '\0';
		strcpy((char *)data, auxdata);
	      }
	      else  
		strcpy((char *)data,pdata);	
	  }
	else
		return(-1);

	return (0);
}

/************************************************************************/
/*				FIND					*/
/*									*/
/*  Description: Search a string in a file and returns the string 	*/
/*		 that follow the one searched (separated by a blank	*/
/*		 space.							*/
/*									*/
/*  Parameters: searchstr -> The string that we search for.		*/
/*									*/
/*		filename  -> Name of the file in wich we are going to 	*/
/*			  search for the string.			*/
/*									*/
/*		data	  -> String pointer, where the function will 	*/
/*			  return the string following search_str in the	*/
/*			  file.						*/
/*									*/
/*  Return Value: 1 if found, 0 otherwise                       	*/
/*									*/
/************************************************************************/
 
int find(char *searchstr, char *filename, char *data)
{
	char path[256];
	char letter, buff[256], *ptr;
	int len, ret, found=0;
	FILE *fd;
	
	sprintf(path,"%s/%s", getenv("PWD"),filename);
	fd=fopen(path,"r");

	if(fd)
	{
	  while(!feof(fd))
	    {
	      ret = fscanf(fd,"%[^\n]%c", buff, &letter);
	      if(!ret)
		{
		  fscanf(fd, "%c", &letter);
		  buff[0] = '\0';
		}
	      if(ptr = strstr(buff, searchstr))
		{
		  ptr += strlen(searchstr);
		  sscanf(ptr,"%s", data);
		  found = 1;
		  break;
		}
	    }
	  fclose(fd);
	}
	return(found);
}

int check_nevents(mccontrol_params *params)
{
  char file[64], events[64];

  sprintf(file,"job%d.txt", params->mcrun); // Name of the logfile
  if(!find("EVENTS",file, events))
    return(0);
  //  cout << "stat = " << params->mcstat << " logged "<< events << endl;  
  if(atoi(events)!=params->mcstat)
    return(0);
  return(1);
}

int get_nevents(mccontrol_params *params)
{
  char file[64], events[64];

  sprintf(file,"job%d.txt", params->mcrun); // Name of the logfile
  if(!find("EVENTS",file, events))
    return(0);
  //  cout << "mccontrol: stat = " << params->mcstat << " logged "<< events << endl;
  return(atoi(events));
}

int check_dbase(mccontrol_params *params)
{
  char file[64], dbase[64];

  sprintf(file,"job%d.txt", params->mcrun); // Name of the logfile
  if(!find("Data Base from",file, dbase))
    return(0);
  if(!strncmp(params->LHCBDBASE, dbase, strlen(params->LHCBDBASE)))
    return(1);
  //  cout << "mccontrol: dbase = " << params->LHCBDBASE << " logged "<< dbase << endl;  
  return(0);
}

int main(int argc, char **argv)
{
  char name[132], host[64]; 
  JobService job;
  mccontrol_params params;
  int ppid;
  int center;
  char aux[32], *ptr;

  if(argc < 2)
  {
      exit(0);
  }
  create_env_serv("mccentre", &center, 'I');
  // KGG; work around for id >= 20 translate ->101
  if (center>100) strcpy(name, MCCenterNames[center-101+20]);
  else strcpy(name, MCCenterNames[center]);
  
#ifdef WIN32
  init_sock();
#endif
  gethostname(host,64);
  if(ptr = strchr(host,'.'))
    *ptr = '\0';
  ppid=getppid();
  create_env_serv("LHCBVERS", job.job_params.LHCBVERS, 'C');
  create_env_serv("LHCBDBASE", job.job_params.LHCBDBASE, 'C');
  create_env_serv("SICBCARDS", job.job_params.SICBCARDS, 'C');
  create_env_serv("QQ_DECAY_FILE", job.job_params.QQ_DECAY_FILE, 'C');
  create_env_serv("QQ_USER_FILE", job.job_params.QQ_USER_FILE, 'C');
  create_env_serv("mcrunfile", job.job_params.mcrunfile, 'C');
  create_env_serv("mcjobdir", job.job_params.mcjobdir, 'C');
  create_env_serv("DGJOBID", job.job_params.mcjobid, 'C');
  create_env_serv("DOMAIN", job.job_params.DOMAIN, 'C');
  create_env_serv("mcstat", &job.job_params.mcstat, 'I');
  create_env_serv("mcseed", job.job_params.mcseed, 'I');
  create_env_serv("mcgene", &job.job_params.mcgene, 'I');
//create_env_serv("mccentre", &job.job_params.mccentre, 'I');
  create_env_serv("mcevtype", &job.job_params.mcevtype, 'I');
  create_env_serv("mcpdf", job.job_params.mcpdf, 'I');
  create_env_serv("mcmimodel", &job.job_params.mcmimodel, 'I');
  create_env_serv("mcevbeg", &job.job_params.mcevbeg, 'I');
  create_env_serv("mcrun", &job.job_params.mcrun, 'I');
  create_env_serv("mcfile", job.job_params.mcfile, 'C');
  create_env_serv("mcsize", &job.job_params.mcsize, 'I');
  create_env_serv("mcexec", job.job_params.mcexec, 'C');
  create_env_serv("SUBMIT_DATE", job.job_params.submit_date_time, 'C');
  create_env_serv("SUBMIT_TIME", aux, 'C');
  strcat(job.job_params.submit_date_time," ");
  strcat(job.job_params.submit_date_time,aux);

  job.job_params.mccentre = center;

  strcpy(job.job_params.state,argv[1]);
  job.job_params.alarm = 0;
  if(argc == 3)
      job.job_params.alarm = 1;
  if(!strcmp(argv[1],"Execution completed"))
    {
        job.job_params.mcstat = get_nevents(&job.job_params);
        cout << job.job_params.mcstat << endl;   
        if(!check_dbase(&job.job_params))
	  {
	    job.job_params.alarm = 1;
	    strcat(job.job_params.state," - LHCBDBASE mismatch");  
	  }
    }
  strcat(name,"/MCCONTROL/JOB");
  sprintf(job.job_id,"%s_%d",host, ppid);
//  cout << "mccontrol: " << job.job_params.state << endl;
  DimClient::sendCommand(name, &job, sizeof(job));
//  if(!strcmp(argv[1],"Execution completed")) {
//     cout <<  job.job_params.mcstat << endl;
//  }
  exit(0);
}


#ifdef WIN32

int getppid() 
{ 
    HANDLE         hProcessSnap = NULL; 
    PROCESSENTRY32 pe32      = {0}; 
    int            bRet      = 0;
	int			   pid;
 
    //  Take a snapshot of all processes in the system. 

	pid = _getpid();

    hProcessSnap = CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, 0); 

    if (hProcessSnap == INVALID_HANDLE_VALUE) 
        return (0); 
 
    //  Fill in the size of the structure before using it. 

    pe32.dwSize = sizeof(PROCESSENTRY32); 
 
    //  Walk the snapshot of the processes, and for each process, 
    //  display information. 

    if (Process32First(hProcessSnap, &pe32)) 
    { 
       do 
       { 
//			cout << "pid = " << pe32.th32ProcessID << endl;
			if(pid == pe32.th32ProcessID)
			{
				cout << "ppid = " << pe32.th32ParentProcessID << endl;
				bRet = pe32.th32ParentProcessID;
				break;
			}
       } 
       while (Process32Next(hProcessSnap, &pe32)); 
    } 
 
    // Do not forget to clean up the snapshot object. 

    CloseHandle (hProcessSnap); 
    return (bRet); 
} 

int init_sock()
{
	WORD wVersionRequested;
	WSADATA wsaData;
	int err;
	static int sock_init_done = 0;

	if(sock_init_done) return(1);
 	wVersionRequested = MAKEWORD( 2, 0 );
	err = WSAStartup( wVersionRequested, &wsaData );

	if ( err != 0 ) {

    	/* Tell the user that we couldn't find a usable */
	    /* WinSock DLL.                                  */

    	return(0);
	}

	/* Confirm that the WinSock DLL supports 2.0.*/
	/* Note that if the DLL supports versions greater    */
	/* than 2.0 in addition to 2.0, it will still return */
	/* 2.0 in wVersion since that is the version we      */
	/* requested.                                        */

	if ( LOBYTE( wsaData.wVersion ) != 2 ||
        HIBYTE( wsaData.wVersion ) != 0 ) {
	    /* Tell the user that we couldn't find a usable */
    	/* WinSock DLL.                                  */

	    WSACleanup( );
    	return(0); 
	}
	sock_init_done = 1;
	return(1);
}


#endif

